/* bumbleshow2.c -- bumbleball simulation with colors and sliders
 *	to control parameters:
 *
 * uses the STRUM instrument -- the three sliders control the
 * "squish" factor, the distortion gain, and the feedback.
 * The motion of the red ball was intended to model my son
 * Daniel's "Bumbleball" toy.  Each quadrant it enters has
 * a separate pitch set
 *
 * It's probably a good idea to say "xrdb < Bumbleshow" to
 * see the intended colors and fonts...
 *
 * BGG
*/

#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/Scale.h>
#include <Xm/Protocols.h>

#include <stdio.h>

#include "/musr/cmix/H/RTsockfuncs.h"
#include "/musr/cmix/H/randfuncs.h"

int theSock;
int RTpid;

float pitches1[4] = { 8.00, 8.02, 8.05, 8.07 };
	float pitches2[4] = { 6.00, 6.01, 6.02, 6.03 };
	float pitches3[4] = { 9.04, 9.07, 9.11, 9.05 };
	float pitches4[4] = { 7.09, 8.09, 9.09, 6.09 };

int squish = 2;
float distgain = 1.0;
float fback = 0.1;

Widget theDraw, topLevel;
Widget bboard, bumbutton;
Widget slidey1;
Widget slidey2;
Widget slidey3;
int xball,yball;

GC gc;
Colormap cmap;
XColor bground,ball,lines,unused;

XtAppContext app;

Atom del_atom; /* use this for shutting down cleanly */


main(argc,argv)
int argc;
char *argv[];
{
	XmString text;
	void GoDraw();
	void Csquish();
	void Cdistgain();
	void Cfback();
	void killem();

	/* start up the instrument */
	RTpid = RTopensocket(0, "CMIX");
	sleep(3); /* allow the process time to start */

	/* open up the socket */
	theSock = RTsock("localhost", 0);

	/* set up the instrument */
	RTsendsock("rtsetparams", theSock, 3, 22050.0, 1.0, 256.0);
	RTsendsock("load", theSock, 1, "STRUM");


	/* now do the window stuff */
	topLevel=XtVaAppInitialize(&app,"Bumbleshow",
		NULL,0,&argc,argv,NULL,NULL);

	/* set up the main window */
	bboard = XtVaCreateManagedWidget("bboard",
		xmBulletinBoardWidgetClass, topLevel,
		XmNwidth, 500, XmNheight, 700, 
		XmNmarginWidth, 0, XmNmarginHeight, 0,NULL);

	/* set up the colors */
	cmap = DefaultColormapOfScreen(XtScreen(bboard));
	XAllocNamedColor (XtDisplay(bboard),cmap,"sandybrown",&bground,&unused);
	XAllocNamedColor (XtDisplay(bboard),cmap,"royalblue",&lines,&unused);
	XAllocNamedColor (XtDisplay(bboard),cmap,"red1",&ball,&unused);
	gc = XCreateGC(XtDisplay(bboard),
		RootWindowOfScreen(XtScreen(bboard)), 0, NULL);

	/* set up the drawing area in the main window */
	theDraw = XtVaCreateManagedWidget("theDraw",
		xmDrawingAreaWidgetClass, bboard,
		XmNwidth, 500, XmNheight, 500,
		XmNbackground, bground.pixel, NULL);

	/* set up the bumble button */
	text = XmStringCreateSimple("bumbleize!");
	bumbutton = XtVaCreateManagedWidget("bumbutton",
		xmPushButtonWidgetClass, bboard,
		XmNlabelString, text,
		XmNx, 100, XmNy, 600, NULL);
	XtAddCallback(bumbutton, XmNactivateCallback, GoDraw, NULL);

	/* set up the sliders YAY TEAM!*/

	slidey1 = XtVaCreateManagedWidget("slidey1",
		xmScaleWidgetClass, bboard,
		XmNx, 250, XmNy, 550, XmNshowValue, True,
		XmNorientation, XmHORIZONTAL,
		XmNmaximum, 10, XmNscaleWidth, 200,
		XmNvalue, squish, XmNscaleHeight, 20, NULL);

	XtAddCallback(slidey1,XmNvalueChangedCallback,Csquish,NULL);
	XtAddCallback(slidey1,XmNdragCallback,Csquish,NULL);

	slidey2 = XtVaCreateManagedWidget("slidey2",
		xmScaleWidgetClass, bboard,
		XmNx, 250, XmNy, 600, XmNshowValue, True,
		XmNorientation, XmHORIZONTAL,
		XmNdecimalPoints, 2,
		XmNmaximum, 1000, XmNscaleWidth, 200,
		XmNvalue, (int)distgain, XmNscaleHeight, 20, NULL);

	XtAddCallback(slidey2,XmNvalueChangedCallback,Cdistgain,NULL);
	XtAddCallback(slidey2,XmNdragCallback,Cdistgain,NULL);

	slidey3 = XtVaCreateManagedWidget("slidey3",
		xmScaleWidgetClass, bboard,
		XmNx, 250, XmNy, 650, XmNshowValue, True,
		XmNorientation, XmHORIZONTAL,
		XmNdecimalPoints, 3,
		XmNmaximum, 1000, XmNscaleWidth, 200,
		XmNvalue, (int)fback, XmNscaleHeight, 20, NULL);

	XtAddCallback(slidey3,XmNvalueChangedCallback,Cfback,NULL);
	XtAddCallback(slidey3,XmNdragCallback,Cfback,NULL);

	tsrand(); /* seeds the random stuff by time-of-day */

	xball = brrand() * 500.0;
	yball = brrand() * 500.0;

	XtManageChild(bboard);
	XtManageChild(theDraw);
	XtRealizeWidget(topLevel);


	/* this is to shut down the process cleanly */
	del_atom = XInternAtom(XtDisplay(topLevel), "WM_DELETE_WINDOW", False);
	XmAddWMProtocolCallback(topLevel, del_atom, killem, NULL);

	XtAppMainLoop(app);
}

void Csquish(w, client, call)
Widget w;
XtPointer client, call;
{
	int ival;

	XmScaleGetValue(slidey1, &ival);
	squish = ival;
}

void Cdistgain(w, client, call)
Widget w;
XtPointer client, call;
{
	int ival;

	XmScaleGetValue(slidey2, &ival);
	distgain = (float)ival/100.0;
	if (distgain == 0.0) distgain = 0.001;
}

void Cfback(w, client, call)
Widget w;
XtPointer client, call;
{
	int ival;

	XmScaleGetValue(slidey3, &ival);
	fback = (float)ival/1000.0;
}


void GoDraw(w, client, call)
Widget w;
XtPointer client, call;
{
	void gobumble();

	/* draw the axes */
	XSetForeground(XtDisplay(theDraw), gc, lines.pixel);
	XDrawLine(XtDisplay(theDraw), XtWindow(theDraw), gc, 250, 0, 250, 500);
	XDrawLine(XtDisplay(theDraw), XtWindow(theDraw), gc, 0, 250, 500, 250);

	/* draw the ball */
	XSetForeground(XtDisplay(theDraw), gc, ball.pixel);
	XFillArc(XtDisplay(theDraw), XtWindow(theDraw), gc, xball, yball,
		10, 10, 0, (360*64));

	/* get the timer going */
	RTtimeit(0.2, gobumble);
}


void gobumble()
{
	int index;
	float note;

	/* erase the ball */
	XSetForeground(XtDisplay(theDraw), gc, bground.pixel);
	XFillArc(XtDisplay(theDraw), XtWindow(theDraw), gc, xball, yball,
		10, 10, 0, (360*64));

	xball = (int)(rrand()*50.0) + xball;
	yball = (int)(rrand()*50.0) + yball;
	if (xball > 500) xball = 490;
	if (xball < 0) xball = 10;
	if (yball > 500) yball = 490;
	if (yball < 0) yball =  10;

	/* redraw the axes */
	XSetForeground(XtDisplay(theDraw), gc, lines.pixel);
	XDrawLine(XtDisplay(theDraw), XtWindow(theDraw), gc, 250, 0, 250, 500);
	XDrawLine(XtDisplay(theDraw), XtWindow(theDraw), gc, 0, 250, 500, 250);

	/* redraw the ball */
	XSetForeground(XtDisplay(theDraw), gc, ball.pixel);
	XFillArc(XtDisplay(theDraw), XtWindow(theDraw), gc, xball, yball,
		10, 10, 0, (360*64));

	XFlush(XtDisplay(theDraw));

	/* ok, decide where the ball is */
	if (xball < 250) { 
		if (yball < 250) {
			index = brrand() * 4.0;
			note = pitches1[index];
		}
		else {
			index = brrand() * 4.0;
			note = pitches2[index];
		}
	}
	else {
		if (yball < 250) {
			index = brrand() * 4.0;
			note = pitches3[index];
		}
		else {
			index = brrand() * 4.0;
			note = pitches4[index];
		}
	}

		RTsendsock("START1", theSock, 14, 0.0, 1.0, note, 1.0, 0.5, distgain, fback, 7.00, 0.0, 1.0, 10000.0, (float)squish, 0.0, 1.0);
}

void killem(w, client, call)
Widget w;
XtPointer client, call;
{
	RTkillsocket(theSock, RTpid);
	sleep(1);
	exit(0);
}

